<?php

namespace Drupal\element_pack\Element;

use Drupal\Core\Render\Attribute\FormElement;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\FormElementBase;

/**
 * Provides a custom button element using a <button> tag.
 *
 * @FormElement("element_pack_button")
 *
 * Unlike the default Drupal button elements, which generate <input type="button"> or
 * <input type="submit"> tags, this custom button element specifically uses a <button> tag.
 * This allows for richer content (e.g., HTML, icons) inside the button, making it more
 * versatile for modern web design.
 *
 * Example:
 * @code
 * $form['my_button'] = [
 *   '#type' => 'element_pack_button',
 *   '#label' => Markup::create('<span>Click Me</span>'),
 *   '#attributes' => [
 *     'class' => ['my-custom-button'],
 *     'onclick' => 'alert("Button clicked!");',
 *     'type' => 'button',
 *   ],
 * ];
 * @endcode
 *
 * Key Features:
 * - Uses a <button> tag instead of <input>, allowing for more flexible and semantic design.
 * - Supports custom attributes, including JavaScript handlers and CSS classes.
 * - Ideal for use cases requiring dynamic content or advanced styling within the button.
 *
 * @see https://www.drupal.org/project/drupal/issues/1671190
 */
#[FormElement('element_pack_button')]
class ButtonElement extends FormElementBase {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#input' => TRUE,
      '#theme' => 'element_pack_button',
      '#label' => '',
      '#pre_render' => [
        [$class, 'preRender'],
      ],
      '#process' => [
        [$class, 'processAjaxForm'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function preRender($element) {
    // Set necessary attributes.
    Element::setAttributes($element, ['id']);

    return $element;
  }

}
